import path from 'path';
import fs from 'fs';
import webpack, { DefinePlugin, ProgressPlugin, container } from 'webpack';
import HtmlWebpackPlugin from 'html-webpack-plugin';
import MiniCssExtractPlugin from 'mini-css-extract-plugin';
import CopyWebpackPlugin from 'copy-webpack-plugin';
import ForkTsCheckerWebpackPlugin from 'fork-ts-checker-webpack-plugin';
import ESLintPlugin from 'eslint-webpack-plugin';
import { RcbConfig } from './getConfig';
import webpackBar from './common/webpackBar';
import utils from './utils';

const ModuleFederationPlugin = container.ModuleFederationPlugin;
function hasTSConfig() {
    return fs.existsSync(path.resolve(process.cwd(), 'tsconfig.json'));
}

function addPlugins(basePlugins: any, plugins?: any) {
    if (plugins) {
        plugins.forEach(function (plugin: HtmlWebpackPlugin | DefinePlugin) {
            basePlugins.push(plugin);
        });
    }

    return basePlugins;
}

export default function getPlugins(rcb: RcbConfig) {
    const basePlugins: any = [
        rcb.moduleFederationPlugin && new ModuleFederationPlugin(rcb.moduleFederationPlugin),
        new CopyWebpackPlugin({
            patterns: [{
                from: path.resolve('public'),
                globOptions: { ignore: ['**/index.html'] }
            }]
        }),
        new webpack.DefinePlugin({
            'process.env': {
                NODE_ENV: JSON.stringify(utils.getEnv())
            }
        }),
        new ESLintPlugin({
            extensions: ['.js', '.jsx', '.es6', '.ts', '.tsx', '.mjs'],
            eslintPath: require.resolve('eslint'),
            resolvePluginsRelativeTo: __dirname,
            baseConfig: utils.requireModule(require.resolve('./eslintrc'))
        }),
        new HtmlWebpackPlugin({
            template: path.resolve('./public/index.html')
        }),
        new ProgressPlugin((c: any, m: any, a: any) => {
            if (m !== 'setup') { // 设置阶段不展示webpack bar
                webpackBar.update(c);
            }
        })
    ];

    if (hasTSConfig()) {
        basePlugins.push(new ForkTsCheckerWebpackPlugin({
            async: true
        }));
    }

    if (utils.isDev()) {
        addPlugins(basePlugins, [
            new webpack.HotModuleReplacementPlugin(),
            new MiniCssExtractPlugin({
                filename: 'css/[name].css'
            })
        ]);
    }

    if (utils.isProduction()) {
        addPlugins(basePlugins, [
            new MiniCssExtractPlugin({
                filename: 'css/[name].[contenthash:8].css'
            })
        ]);
    }

    if (rcb.webpackPluginConfig && rcb.webpackPluginConfig.length > 0) {
        return [basePlugins, ...rcb.webpackPluginConfig];
    }

    return basePlugins.filter(Boolean);
};
